﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Windows;
using ThinkGeo.MapSuite.Core;

namespace ThikGeo_Contour_Problem
{
    public class Point
    {
        public double XCoord { get; set; }
        public double YCoord { get; set; }
    }

    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private RectangleShape _extent;

        private string _gridPath;

        public MainWindow()
        {
            InitializeComponent();
            wpfMap.Loaded += WpfMapLoaded;
        }

        private void CreateGridCellFile(Dictionary<PointShape, double> points, double rowCount, double columnCount,
                                        double noDataValue, double power)
        {
            double cellSize = Math.Min(_extent.Width/columnCount, _extent.Height/rowCount);
            var gridDefinition = new GridDefinition(_extent, cellSize, noDataValue, points);
            FileStream writer = null;
            try
            {
                writer = new FileStream(_gridPath, FileMode.Create);
                GridFeatureSource.GenerateGrid(gridDefinition,
                                               new InverseDistanceWeightedGridInterpolationModel(power, cellSize*4),
                                               writer);
            }
            finally
            {
                if (writer != null) writer.Close();
            }
        }

        private void WpfMapLoaded(object sender, RoutedEventArgs e)
        {
            try
            {
                _gridPath = Path.Combine(TempFolderHelper.Folder, string.Format(@"{0}.grid", 1));

                List<Point> points = ReadPointsFromCSV();
                _extent = new RectangleShape(317607, 181891, 319607, 180891);

                CreateGridCellFile(points.ToDictionary(key => new PointShape(key.XCoord, key.YCoord), v => 1.0), 10, 10,
                                   -1, 2);


                var isoLineBreaks = new List<double> {10};
                var gridFeatureSource = new GridFeatureSource(_gridPath);
                gridFeatureSource.Open();

                Collection<Feature> isoFeatures = IsoLineLayer.GetIsoFeatures(gridFeatureSource.GenerateGridMatrix(),
                                                                              isoLineBreaks, "GridCellValue",
                                                                              IsoLineType.ClosedLinesAsPolygons);
                isoFeatures.ToList().ForEach(f => f.MakeValid());
            }
            catch (Exception ex)
            {
            }
        }

        private List<Point> ReadPointsFromCSV()
        {
            string filename = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location) + @"\data\Points.csv";
            var points = new List<Point>();
            using (var rd = new StreamReader(filename))
            {
                while (!rd.EndOfStream)
                {
                    string[] splits = rd.ReadLine().Split(',');
                    points.Add(new Point
                                   {
                                       XCoord = Convert.ToDouble(splits[0]), YCoord = Convert.ToDouble(splits[1])
                                   });
                }
            }
            return points;
        }

        #region Nested type: TempFolderHelper

        public static class TempFolderHelper
        {
            public static String Folder
            {
                get
                {
                    string tempPath = Path.Combine(Environment.GetEnvironmentVariable("TEMP"), "ThinkGeo");
                    if (!Directory.Exists(tempPath)) Directory.CreateDirectory(tempPath);
                    return tempPath;
                }
            }
        }

        #endregion
    }
}